"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GameData = void 0;
const constant_1 = require("@skeldjs/constant");
const protocol_1 = require("@skeldjs/protocol");
const Networkable_1 = require("../Networkable");
const PlayerInfo_1 = require("../misc/PlayerInfo");
const component_1 = require("./component");
const events_1 = require("../events");
const endgame_1 = require("../endgame");
/**
 * Represents a room object containing data about players.
 *
 * See {@link GameDataEvents} for events to listen to.
 */
class GameData extends Networkable_1.Networkable {
    constructor(room, spawnType, netId, ownerid, flags, data) {
        super(room, spawnType, netId, ownerid, flags, data);
        this.players || (this.players = new Map);
    }
    Awake() {
        for (const [, player] of this.room.players) {
            if (player.playerId)
                this.add(player.playerId);
        }
    }
    getComponent(component) {
        if (this.spawnType === constant_1.SpawnType.GameData) {
            if (component === GameData) {
                return this.components[0];
            }
            if (component === component_1.VoteBanSystem) {
                return this.components[1];
            }
        }
        return super.getComponent(component);
    }
    get owner() {
        return super.owner;
    }
    /**
     * Resolve some player identifier to a definite player info object.
     * @param resolvable The player identifier to resolve.
     * @returns A definite player info object.
     */
    resolvePlayerData(resolvable) {
        const resolved = this.room.resolvePlayerId(resolvable);
        if (resolved === undefined)
            return undefined;
        return this.players.get(resolved);
    }
    Deserialize(reader, spawn = false) {
        if (!this.players)
            this.players = new Map;
        while (reader.left) {
            const [playerId, preader] = reader.message();
            const player = this.players.get(playerId);
            if (player) {
                player.Deserialize(preader);
            }
            else {
                const player = PlayerInfo_1.PlayerInfo.Deserialize(preader, this, playerId);
                this.players.set(player.playerId, player);
            }
        }
    }
    Serialize(writer, spawn = false) {
        let flag = false;
        for (const [playerId, player] of this.players) {
            if (((1 << playerId) & this.dirtyBit) || spawn) {
                writer.begin(player.playerId);
                writer.write(player);
                writer.end();
                flag = true;
            }
        }
        this.dirtyBit = 0;
        return flag;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.SetTasks:
                this._handleSetTasks(rpc);
                break;
        }
    }
    /**
     * Make the player data dirty and update on the next FixedUpdate.
     * @param resolvable The player to make dirty.
     */
    markDirty(resolvable) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            this.dirtyBit |= 1 << player.playerId;
        }
    }
    async _handleSetTasks(rpc) {
        const playerData = this.players.get(rpc.playerId);
        if (playerData) {
            const oldTasks = playerData.taskStates;
            this._setTasks(playerData, rpc.taskIds.map(taskType => new PlayerInfo_1.TaskState(taskType, false)));
            const playerTasks = playerData.taskStates;
            const ev = await this.emit(new events_1.GameDataSetTasksEvent(this.room, this, playerData, oldTasks, playerTasks));
            playerData.taskStates = ev.alteredTasks;
            if (ev.alteredTasks !== playerTasks) {
                this._rpcSetTasks(playerData, playerData.taskStates);
            }
        }
    }
    _setTasks(player, taskStates) {
        player.taskStates = taskStates;
    }
    _rpcSetTasks(player, taskStates) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetTasksMessage(player.playerId, taskStates.map(state => state.taskType))));
    }
    /**
     * Set the tasks of a player.
     * @param player The player to set the tasks of.
     * @param taskTypes The task types to give to the player, see {@link TaskType}.
     * @example
     *```typescript
     * room.gamedata.setTasks(player, [
     *   TheSkeldTask.ReactorUnlockManifolds,
     *   TheSkeldTask.ElectricDownloadData,
     *   TheSkeldTask.ShieldsPrimeShields,
     *   TheSkeldTask.NavigationDownloadData
     * ]);
     * ```
     */
    async setTasks(player, taskTypes) {
        const playerData = this.resolvePlayerData(player);
        if (playerData) {
            const oldTasks = playerData.taskStates;
            this._setTasks(playerData, taskTypes.map(taskType => new PlayerInfo_1.TaskState(taskType, false)));
            const ev = await this.emit(new events_1.GameDataSetTasksEvent(this.room, this, playerData, oldTasks, playerData.taskStates));
            this._setTasks(playerData, ev.alteredTasks);
            if (playerData.taskStates !== oldTasks) {
                this._rpcSetTasks(playerData, playerData.taskStates);
                this.markDirty(playerData);
            }
        }
    }
    /**
     * Mark a player's task as complete.
     * @param resolvable The player of the tasks to mark complete.
     * @param taskIdx The index of the player's tasks to mark complete.
     * @example
     *```typescript
     * // Complete all of a player's tasks.
     * for (let i = 0; i < player.playerInfo.tasks.length; i++) {
     *   room.gamedata.completeTask(player, i);
     * }
     * ```
     */
    completeTask(resolvable, taskIdx) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            const task = player.taskStates[taskIdx];
            if (task) {
                task.completed = true;
            }
            else {
                player.taskStates[taskIdx] = new PlayerInfo_1.TaskState(0, true);
            }
            let totalTasks = 0;
            let completeTasks = 0;
            const taskStates = new Map;
            for (const [, playerInfo] of this.players) {
                if (!playerInfo.isDisconnected && !playerInfo.isImpostor) {
                    taskStates.set(playerInfo.playerId, playerInfo.taskStates);
                    for (let i = 0; i < playerInfo.taskStates.length; i++) {
                        totalTasks++;
                        if (playerInfo.taskStates[i].completed) {
                            completeTasks++;
                        }
                    }
                }
            }
            if (totalTasks > 0 && completeTasks >= totalTasks) {
                this.room.registerEndGameIntent(new endgame_1.EndGameIntent(endgame_1.AmongUsEndGames.TasksComplete, constant_1.GameOverReason.HumansByTask, {
                    totalTasks,
                    completeTasks,
                    taskStates
                }));
            }
        }
    }
    /**
     * Add a player to player data.
     * @param playerId The player ID of the player to add.
     * @example
     *```typescript
     * // Get an available player ID and add it to the gamedata.
     * const playerId = room.getAvailablePlayerID();
     * room.gamedata.add(playerId);
     * ```
     */
    async add(playerId) {
        const playerInfo = PlayerInfo_1.PlayerInfo.createDefault(this, playerId);
        this.players.set(playerId, playerInfo);
        const ev = await this.emit(new events_1.GameDataAddPlayerEvent(this.room, this, playerInfo));
        if (ev.reverted) {
            this.players.delete(playerId);
        }
        else {
            this.markDirty(playerId);
        }
        return playerInfo;
    }
    /**
     * Remove player data from the game data.
     * @param resolvable The player to remove.
     */
    async remove(resolvable) {
        const playerInfo = this.resolvePlayerData(resolvable);
        if (playerInfo) {
            const wasMarked = this.dirtyBit & (1 << playerInfo.playerId);
            if (wasMarked) {
                this.dirtyBit ^= 1 << playerInfo.playerId;
            }
            this.players.delete(playerInfo.playerId);
            const ev = await this.emit(new events_1.GameDataRemovePlayerEvent(this.room, this, playerInfo));
            if (ev.reverted) {
                this.players.set(playerInfo.playerId, playerInfo);
                this.markDirty(playerInfo.playerId);
            }
        }
    }
}
exports.GameData = GameData;
//# sourceMappingURL=GameData.js.map